@@ -20,34 +20,37 @@
 
 #include "typesdialog.h"
 
-
 #include "edittype.h"
 #include "ui_typeswidget.h"
 #include "../kig/kig_part.h"
 #include "../misc/guiaction.h"
 #include "../misc/object_constructor.h"
-
-#include <kfiledialog.h>
-#include <kiconloader.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kpushbutton.h>
-#include <KHelpClient>
-
-#include <qbytearray.h>
-#include <qevent.h>
-#include <qfile.h>
-#include <qlayout.h>
-#include <qmenu.h>
-#include <qstringlist.h>
+#include "../kig/kig_document.h"
 
 #include <algorithm>
 #include <iterator>
+
 #include <QDialogButtonBox>
-#include <KConfigGroup>
+#include <QFileDialog>
+#include <QMenu>
 #include <QPushButton>
+#include <QStandardPaths>
 #include <QVBoxLayout>
 
+#include <KConfigGroup>
+#include <KHelpClient>
+#include <KIconLoader>
+#include <KMessageBox>
+
+#ifdef WITH_GEOGEBRA
+#include "../geogebra/geogebratransformer.h"
+
+#include <QDebug>
+#include <QXmlQuery>
+
+#include <KZip>
+#endif //WITH_GEOGEBRA
+
 static QString wrapAt( const QString& str, int col = 50 )
 {
   QStringList ret;
@@ -461,7 +464,7 @@ void TypesDialog::exportType()
       types.push_back( macro );
   }
   if (types.empty()) return;
-  QString file_name = KFileDialog::getSaveFileName( QUrl("kfiledialog:///macro"), i18n("*.kigt|Kig Types Files\n*|All Files"), this, i18n( "Export Types" ) );
+  QString file_name = QFileDialog::getSaveFileName( this, i18n( "Export Types" ), QStandardPaths::writableLocation( QStandardPaths::DocumentsLocation ), i18n("*.kigt|Kig Types Files\n*|All Files") );
   if ( file_name.isNull() )
     return;
   QFile fi( file_name );
@@ -475,15 +478,27 @@ void TypesDialog::exportType()
 
 void TypesDialog::importTypes()
 {
-  QStringList file_names =
-    KFileDialog::getOpenFileNames( QUrl("kfiledialog:///importTypes"), i18n("*.kigt|Kig Types Files\n*|All Files"), this, i18n( "Import Types" ));
+  //TODO : Do this through MIME types
+  QStringList toolFilters;
+  toolFilters << i18n( "*.kigt|Kig Types Files" );
+#ifdef WITH_GEOGEBRA
+  toolFilters << i18n( "*.ggt|Geogebra Tool Files" );
+#endif //WITH_GEOGEBRA
+  toolFilters << i18n( "*|All Files" );
+  QStringList file_names = QFileDialog::getOpenFileNames( this, i18n( "Import Types" ), QStandardPaths::writableLocation( QStandardPaths::DocumentsLocation ), toolFilters.join( QLatin1String( "\n" ) ) );
 
   std::vector<Macro*> macros;
-
   for ( QStringList::const_iterator i = file_names.constBegin();
         i != file_names.constEnd(); ++i)
   {
     std::vector<Macro*> nmacros;
+#ifdef WITH_GEOGEBRA
+    if( i->endsWith( ".ggt" ) )// The input file is a Geogebra Tool file..
+    {
+      loadGeogebraTools( *i, macros, mpart );
+      continue;
+    }
+#endif //WITH_GEOGEBRA
     bool ok = MacroList::instance()->load( *i, nmacros, mpart );
     if ( ! ok )
       continue;
@@ -560,3 +575,51 @@ void TypesDialog::typeListContextMenu( const QPoint& pos )
 
   popup->exec( mtypeswidget->typeList->viewport()->mapToGlobal( pos ) );
 }
+
+#ifdef WITH_GEOGEBRA
+bool TypesDialog::loadGeogebraTools( const QString& sFrom, std::vector<Macro*>& vec, KigPart& /*kigpart*/ )
+{
+  KZip geogebraFile( sFrom );
+
+  if ( geogebraFile.open( QIODevice::ReadOnly ) )
+  {
+    const KZipFileEntry* geogebraXMLEntry = dynamic_cast<const KZipFileEntry*>( geogebraFile.directory()->entry( "geogebra_macro.xml" ) );
+
+    if ( geogebraXMLEntry )
+    {
+      KigDocument * document = new KigDocument();
+      QXmlNamePool np;
+      QXmlQuery geogebraXSLT( QXmlQuery::XSLT20, np );
+      const QString encodedData = QString::fromUtf8( geogebraXMLEntry->data().constData() );
+      QFile queryDevice( ":/kig/geogebra/geogebra.xsl" );
+      GeogebraTransformer ggttransformer( document, np );
+
+      queryDevice.open( QFile::ReadOnly );
+      geogebraXSLT.setFocus( encodedData );
+      geogebraXSLT.setQuery( &queryDevice );
+      geogebraXSLT.evaluateTo( &ggttransformer );
+      queryDevice.close();
+
+      const size_t nmacros = ggttransformer.getNumberOfSections();
+
+      for( size_t i = 0; i < nmacros; i++ )
+      {
+        const GeogebraSection f = ggttransformer.getSection( i );
+        ObjectHierarchy hrchy( f.getInputObjects(), f.getOutputObjects() );
+        MacroConstructor* ctor = new MacroConstructor( hrchy, ggttransformer.getSection( i ).getName(), ggttransformer.getSection( i ).getDescription() );
+        ConstructibleAction* act = new ConstructibleAction( ctor, 0 );
+
+        Macro* newmacro = new Macro( act, ctor );
+        vec.push_back( newmacro );
+      }
+    }
+  }
+  else
+  {
+    qWarning() << "Failed to open zip archive";
+    return false;
+  }
+
+  return true;
+}
+#endif //WITH_GEOEBRA
